######################################################################
############                   FUNCTIONS                  ############
######################################################################

library(mvtnorm)


######### t functions #######


t.logdens.unnorm <-  function(xvec,mu.t, sigma.t, df.t){
### unnormalized density of a non-central multivariate t ###

P.x <- length(xvec)
sq.term <- t(xvec - mu.t) %*% solve(sigma.t) %*% (xvec - mu.t)
kern.t <- -0.5*(df.t + P.x)*log(1 + as.numeric(sq.term)/(df.t))
return(kern.t)
}


#########  posteriors #########


## re-write fn with YY, XX

 b.post <- function(b){

 gpar.inv <- 1/gpar
 lambda <- 0.5

 n <- nrow(YY)
 pp <- ncol(XX)

V.inv <- gpar.inv * ( t(as.matrix(XX)) %*% as.matrix(XX) + lambda *diag(ncol(XX))) ## new prior

lnum <- sum( YY[,1] * as.matrix(XX)%*% b) -0.5 *t(b) %*% V.inv %*% b  + 0.5 * determinant(V.inv, log = T)$modulus[1]

lden <- 0
for(i in 1:n){
lden <- lden + n.binom  *  log(1 + exp(as.matrix(XX)[i,] %*% b))
}

return(lnum - lden)
}




######################################################################


setnum <- 1

yname <- paste("n75.p100.set", setnum, ".",sep="")

gpar.is <- 0.25 ## for the importance trial density (later mixture) (g)
gpar <- 1  ## for the ACTUAL prior (c_0)


XX <- as.matrix(read.table(paste(yname,"txt",sep=""))[,-c(1:2)]) 
YY <- as.matrix(read.table(paste(yname,"txt",sep=""))[,c(1:2)])
n.binom <- 1

outfilename <- paste(yname,"g",gpar, ".samps",sep="")
betfilename <- paste(yname,"bets.g",gpar,".out",sep="")

blen <- ncol(XX)
startval <- rep(1,blen)   
TotSam <- 1000

## importance sampling using t-distribution
## use t- with posterior mean and covariance 


## based on "normal" posterior cov
t.cov <- ( (gpar.is)* diag(blen))
t.mean <- rep(0, blen) 



## choosing gpar.is too large here may cause problems? 

t.df <- 3

for(numtimes in 1:100){ # 100 such samples

logIS.wt <- rep(0, TotSam)

t.meanmat <- matrix(rep(t.mean,TotSam),TotSam, blen,byrow=T )

cat("sample ", numtimes,"\n")

res1.new <- t.meanmat  + rmvt(TotSam, sigma=t.cov, df=t.df)

write("", outfilename, append=F)

for(i in 1:TotSam){
 logIS.wt[i] <- b.post(res1.new[i,]) - t.logdens.unnorm(res1.new[i,], t.mean, t.cov, t.df)

 write(round(c(res1.new[i,],logIS.wt[i]),3), outfilename, ncolumns=(blen+1), append=T)
}


Max.lwt <- max(logIS.wt)
E.log <- log(sum(exp(logIS.wt - rep(Max.lwt, TotSam))) ) + Max.lwt
IS.wt <- exp(logIS.wt - rep(E.log,TotSam))  ## importance weights

beta.est <- rep(0,blen)

for(i in 1:blen){
 beta.est[i] <- sum(IS.wt * res1.new[,i])
}

##  write estimate to separate file, to get bias and var of estimate (include    gpar)

write(c(numtimes, gpar,gpar.is, round(beta.est,4)),file=betfilename, ncolumns=(blen+3),append=T )
}
